import { ZodSchema, z } from 'zod';
import { Client } from '@notionhq/client';
import { Root } from 'mdast';
import { Result, Ok } from 'neverthrow';

type NotionSourceOptions<T> = {
    properties: T;
    client: Client;
    databaseId: string;
};
type NotionProperty<PropertySchema extends ZodSchema> = {
    propertyName: string;
    schema: PropertySchema;
    fallback: unknown | undefined;
};
type SchemaOutputs<T extends Record<string, NotionProperty<ZodSchema>>> = {
    [P in keyof T]: ExtractOk<z.infer<T[P]["schema"]>>;
} & {
    id: string;
};
type FetchPostsOptions = {
    content: boolean;
    /**
     * If set to true, missing fields in posts will not throw, but instead
     * those posts will be not returned
     */
    skipMissingFields: boolean;
    /**
     * @default true
     */
    allowMissingBlocktypes: boolean;
};
type ExtractOk<T> = T extends Ok<infer U, unknown> ? U : never;
type TransformedPost<T extends Record<string, NotionProperty<ZodSchema>>> = SchemaOutputs<T> & {
    blocks: Root;
};
type NotionSource<T extends Record<string, NotionProperty<ZodSchema>>> = {
    fetchPosts: (options: FetchPostsOptions) => Promise<Result<Array<SchemaOutputs<T> | TransformedPost<T>>, unknown>>;
    getPostContents: (postId: string, options: Partial<Pick<FetchPostsOptions, "allowMissingBlocktypes">>) => Promise<Result<Root, unknown>>;
};

/**
 * Known problems;
 * - nested list items not transformed correctly.
 */

declare function createNotionSource<T extends Record<string, NotionProperty<ZodSchema>>>(options: NotionSourceOptions<T>): NotionSource<T>;

export { createNotionSource };
